from django.db import models
import simplejson
from time import localtime, strftime
from datetime import datetime, timedelta
import re
from django_thermostat.utils import gen_comparing_time
from django_thermometer.temperature import read_temperatures
#from django_thermostat.mappings import get_mappings
from django.db.models import Avg
import logging
from django.conf import settings

logger = logging.getLogger("thermostat")
logger.setLevel(settings.LOG_LEVEL)


class Thermometer(models.Model):

    tid = models.CharField(max_length=30, unique=True)
    caption = models.CharField(max_length=30, null=True, blank=True, unique=True)
    is_internal_reference = models.NullBooleanField(unique=True)
    is_external_reference = models.NullBooleanField(unique=True)
    
    def __unicode__(self):
        return u"%s" % self.caption if self.caption is not None else self.tid

    def read(self, ):
        return read_temperatures(self.tid)[self.tid]["celsius"]


class Context(models.Model):
    confort_temperature = models.DecimalField(default=22, decimal_places=2, max_digits=4)
    economic_temperature = models.DecimalField(default=18, decimal_places=2, max_digits=4)
    tuned_temperature = models.DecimalField(null=True, blank=True, decimal_places=2, max_digits=4)

    heat_on = models.BooleanField(default=False)
    manual = models.BooleanField(default=True)
    flame = models.BooleanField(default=False)

    def to_json(self, ):

        return simplejson.dumps({
            "status": "ON" if self.heat_on else "OFF",
            "confort": self.confort_temperature,
            "economic": self.economic_temperature,
            "manual": self.manual,
            "flame": self.flame,
            "tuned": self.tuned_temperature,
            "time": "%s" % strftime("%H:%M:%S", localtime()),
        })


class Day(models.Model):
    name = models.CharField(max_length=10)
    value = models.CharField(max_length=3)

    def __unicode__(self, ):
        return u"%s" % self.name

    def to_pypelib(self, ):
        return u"(current_day_of_week = %s)" % self.value


class TimeRange(models.Model):
    start = models.TimeField()
    end = models.TimeField()

    def __unicode__(self, ):
        return u"%s - %s" % (self.start, self.end)

    def start_to_comparing(self, ):
        return u"%s" % gen_comparing_time(
            self.start.hour,
            self.start.minute,
            self.start.second,
        )

    def end_to_comparing(self, ):
        return u"%s" % gen_comparing_time(
            self.end.hour,
            self.end.minute,
            self.end.second,
        )

    def to_pypelib(self, ):
        #((current_time > %f) && (current_time < %f))
        return u"((current_time > %s ) && (current_time < %s))" % (
            self.start_to_comparing(),
            self.end_to_comparing())


TEMP_CHOICES = (
    ("tune_to_confort", "Confort"),
    ("tune_to_economic", "Economic"),

    ("salon_on", "Subir salon"),
    ("salon_off", "Bajar salon"),

    ("pasillo_off", "Bajar pasillo"),
    ("pasillo_on", "Subir pasillo"),

    ("cuarto_oeste_off", "Bajar cuarto oeste"),
    ("cuarto_oeste_on", "Subir cuarto oeste"),

    ("cuarto_este_off", "Bajar cuarto este"),
    ("cuarto_este_on", "Subir cuarto este"),

    ("a_lights_on", "Subir grupo A"),
    ("a_lights_off", "Bajar grupo A"),
    
    ("set_heater_off", "Apagar caldera"),
    ("set_heater_on", "Encender caldera"),
)


COND_CHOICES = (
    ("is_at_night", "Is at night"),
    ("current_external_temperature", "Current external temperature"),
    ("current_internal_temperature", "Current internal temperature"),
)

OPERATOR_CHOICES = (
    ("=", "="),
    ("<", "<"),
    (">", ">"),
    (">=", ">="),
    ("<=", "<="),
    
)

class Conditional(models.Model):
    statement = models.CharField(max_length=60,choices=COND_CHOICES)
    operator = models.CharField(max_length=2,choices=OPERATOR_CHOICES)
    statement2 = models.CharField(max_length=60, choices=COND_CHOICES, null=True, blank=True)
    value = models.CharField(max_length=10, null=True, blank=True)
    ocurred = models.BooleanField(default=False)
    
    
    def __unicode__(self):
        return self.to_pypelib()
    
    def to_pypelib(self):
#        from django_thermostat.mappings import get_mappings
#        if not self.statement2 is None:
#            self.value = get_mappings()[self.statement2]()
        
        return u"(%s %s %s)" % (
            self.statement, 
            self.operator,
            self.value if self.statement2 is None else self.statement2)
        
    def save(self):
        if self.statement2 is None and self.value == "":
            raise AttributeError("Either statment2 or value must not be none")
        super(Conditional, self).save()


class Rule(models.Model):

    days = models.ManyToManyField(Day, null=True, blank=True)
    ranges = models.ManyToManyField(TimeRange, null=True, blank=True)
    conditionals = models.ManyToManyField(Conditional, null=True, blank=True)
    action = models.CharField(max_length=25, choices=TEMP_CHOICES, default="economic_temperature")
    active = models.BooleanField(default=True)
    thermostat = models.BooleanField(default=False)

    def __unicode__(self, ):
        return "[%s] therm: %s; days: %s; time ranges: %s; conditionals: %s; action: %s" % (
            self.active,
            self.thermostat,
            self.days.all(),
            self.ranges.all(),
            self.conditionals.all(),
            self.action,
        )

    def to_pypelib(self, ):
        if self.thermostat:
            out = "if (heater_manual = 0 ) && "
        else:
            out = "if "
        days = self.days.all()
        ranges = self.ranges.all()
        conds = self.conditionals.all()
        
        if days.count():
            out = "%s (" % out
            for day in days:
                out = "%s %s || " % (out, day.to_pypelib())
            out = re.sub("\|\|\s$", " ) &&", out)

        if ranges.count():
            out = "%s (" % out
            for trang in ranges.all():
                out = "%s %s || " % (out,  trang.to_pypelib())

            out = re.sub("\|\|\s$", ") &&", out)
        
        if conds.count():
            out = "%s (" % out
            for c in conds:
                out = "%s %s || " % (out, c.to_pypelib())
                
            out = re.sub("\|\|\s$", ") &&", out)
        out = re.sub("&&$", "", out)    
        if ranges.count() == 0 and days.count() == 0 and conds.count() == 0:
            out = "%s 1 = 1 " % out
        
        return "%s then accept nonterminal do %s" % (out, self.action)


class ThermometerDataManager(models.Manager):


    @staticmethod
    def get_last_n_days(n):
        """
        Method NOT returning QuerySet
        """
        ffin = datetime.utcnow().replace(minute=0, second=0)
        fini = ffin - timedelta(days=n)
        data = {}

        for d in ThermometerData.objects.filter(timestamp__gt=fini, timestamp__lt=ffin):
            if d.thermometer.caption not in data:
                data[d.thermometer.caption] = {}
            data[d.thermometer.caption][d.timestamp.strftime('%s')] = d.value
        return data

    @staticmethod
    def get_last_n_weeks(n):
        """
        Method NOT returning QuerySet
        """
        data = {}
        for i in reversed(range(7*n)):
            ffin = datetime.utcnow().replace(hour=0, minute=0, second=0) - timedelta(days=i)
            fini = ffin - timedelta(days=1)

            logger.debug("inteval: %s - %s" % (fini, ffin))
            for therm in Thermometer.objects.all():
                if therm.caption not in data:
                    data[therm.caption] = {}
                d = ThermometerData.objects.filter(
                    thermometer=therm,
                    timestamp__gt=fini,
                    timestamp__lt=ffin).aggregate(Avg('value'))
                data[therm.caption][fini.strftime('%s')] = d['value__avg']
                logger.debug("thermomentro: %s, data: %s" % (therm.id, d['value__avg']))
        return data

    @staticmethod
    def get_last_n_months(n):
        pass
    @staticmethod
    def get_last_year(self):
        pass


class ThermometerData(models.Model):
    objects = ThermometerDataManager()
    thermometer = models.ForeignKey(Thermometer)
    timestamp = models.DateTimeField(auto_now_add=True)
    value = models.DecimalField(max_digits=6, decimal_places=2)
    
    def __unicode__(self):
        return u"%s, %s: %0.2f" % (self.thermometer, self.timestamp, self.value)    



import requests, logging, re

from django.core.management.base import BaseCommand, CommandError
from time import localtime, time, mktime

import logging
from django.conf import settings


logger = logging.getLogger("thermostat")
logger.setLevel(settings.LOG_LEVEL)


class Command(BaseCommand):
    args = ''
    help = 'Parse de date stats file to retrieve the statistics aggregation for the last N minutes.'

    def _time_range(self, length):
        current = time()
        return [float(current - length * 60), float(current)]

    def handle(self, *args, **options):
#        logging.basicConfig(level=logging.DEBUG)
        
        if len(args) != 1:
            self.stderr.write("Please send argument of how many minutes")
            exit(1)
        with open(settings.FLAME_STATS_PATH) as f:
            contents = f.readlines()
        t_range = self._time_range(int(args[0]))
#        logging.debug("Time range %s" % t_range)
        cont = 0
        
        last_start = None
        data = []
        for line in contents:
            cont = cont + 1
            m = re.search("(\d+\.\d+)\n$", line)
            
            if m is None:
                logging.warn("Format of line %d not correct, cannot find time from epoch at the end" % cont)
                continue
            time = float(m.groups(0)[0])
            
            m = re.search("^(ON|OFF)", line)
            if m is None:
                logging.warn("Format of line %d not correct, cannot find ON|OFF action at the beginning" % cont)
                continue
            action = m.groups(0)[0]
            
            if time < t_range[0] or time > t_range[1]:
                logging.debug("Dejamos la linea %d fuera porque se sale del rango pedido" % cont)
                #initialize the last_start so we know it even if it occur before the time range 
                #if the line is OFF we put last_start = None so the state of the flame is off
                if action == "ON":
                    last_start = time
                else:
                    last_start = None 
                continue
            
            data.append([action, time])
        #print data 
        #print "last_Start: %s " % last_start
        last_heating_period = None
        total_heating_period = 0
        for action, time in data:
            if action == "ON":
                last_start = time
                last_heating_period = None
            if action == "OFF" and last_start is not None:
                #TODO que pasa si last_start es None
                last_heating_period = time - last_start 
                last_start = None
                
            total_heating_period = int(total_heating_period) + int((last_heating_period if last_heating_period is not None else 0))
            """
            logging.debug("action: %s time: %d; %d %d %d" % (
                        action, 
                        time,
                        last_heating_period if last_heating_period is not None else 0,
                        last_start if last_start is not None else 0,
                        total_heating_period))
            """
        #si there is no data (for example: every line was out of time range, we need to know
        #the status of the flame. Therefore we use the last_start, which is set above, when the time range uis checked
        if len(data) == 0 and last_start is not None:
   #         print "como todas las lineas son anteriores al periodo y esta arrancado desde antes, le meto el tiempo desde el principio del periodo"
            total_heating_period = int(args[0]) * 60
   #         print total_heating_period
        #if the first action in data is OFF, means the flame was on by the starting of the time range
        #it is needed to add this time
        if len(data) and data[0][0] == "OFF":
  #          print data[0][1]
    #        print "como la pimera linea es OFF tenemos que add el tiempo desde el principip del periodo hasta el OFF, ates: %s" % total_heating_period
             
            total_heating_period = int(total_heating_period) + int(data[0][1] - t_range[0])
     #       print "despuest %s " % total_heating_period
        
        #if the last action is ON, need to add the time from the instance of that last action,
        #to the end of the range
        if len(data) and data[len(data)-1][0] == "ON":
            total_heating_period = int(total_heating_period) + int(t_range[1] - data[0][1])
        
        try:
            total_seconds = int(args[0]) * 60
            print("absolute:%d percent:%.2f" % (total_heating_period, (100 * total_heating_period ) / total_seconds))
        except Exception as er:
            logger.error("Exception while trying to return the value: %s " % er)
            



import logging
from django.conf import settings
#logging.basicConfig(format='%(asctime)s [%(filename)s:%(lineno)d] %(levelname)s: %(message)s', datefmt='%d/%m/%Y %I:%M:%S %p', level=logging.INFO)


class Logger():

	@staticmethod
	def getLogger():
		#Simple wrapper. Ensures logging is always correctly configured (logging.basicConfig is executed)
		logger = logging.getLogger("thermostat")
        logger.setLevel(settings.LOG_LEVEL)
        return logger



import requests
from django.conf import settings
from hautomation_restclient.cmds import pl_switch
from django_thermostat.models import Context, Thermometer
from django.core.urlresolvers import reverse
from time import strftime, localtime, mktime, strptime
import os, logging, simplejson

logger = logging.getLogger("thermostat")
logger.setLevel(settings.LOG_LEVEL)


def current_external_temperature(mo=None):
    try:

        if not settings.LIST_THERMOMETERS_API is None:
            ret = requests.get("http://localhost%stemperatures=True" % reverse("temperatures"))
            therms = ret.json()
        else:
            therms = simplejson.loads(read_temperatures())

        for therm in therms:
            data = therms[therm]

            if data["is_external"]:
                d = float(data["temp"]["celsius"])
                logger.debug("External temperature: %s" % d)
                return d
        logger.debug("No external thermometer found")
    except Exception as et:
        logger.error(et)


def current_internal_temperature(mo=None):
    try:

        if not settings.LIST_THERMOMETERS_API is None:
            ret = requests.get("http://localhost%stemperatures=True" % reverse("temperatures"))
            therms = ret.json()
        else:
            therms = simplejson.loads(read_temperatures())

        for therm in therms:
            data = therms[therm]
      
            if data["is_internal"]:
                t = float(data["temp"]["celsius"])
                logger.debug("Internal temp: %s" % t)
                return t
        logger.debug("No internal thermometer found")
    except Exception as et:
        logger.error(et)
    

def confort_temperature(mo=None):
    try:
        ctxt = Context.objects.get()

        t = float(ctxt.confort_temperature) + settings.HEATER_MARGIN \
            if ctxt.flame else ctxt.confort_temperature
        logger.debug("Comfort temp: %s" % t)
        return t
    except Exception as et:
        logger.error(et)


def economic_temperature(mo=None):
    try:
        ctxt = Context.objects.get()

        t = float(ctxt.economic_temperature) + settings.HEATER_MARGIN \
            if ctxt.flame else ctxt.economic_temperature
        logger.debug("Economic temp:  %s" % t)
        return t
    except Exception as et:
        logger.error(et)

def tuned_temperature(mo=None):
    try:
        ctxt = Context.objects.get()

        t = float(ctxt.tuned_temperature) + settings.HEATER_MARGIN \
            if ctxt.flame elsectxt.tuned_temperature
        logger.debug("Tunned temp: %s" % t)
        return t
    except Exception as et:
        logger.error(et)

def tune_to_confort(mo=None):
    try:
        ctxt = Context.objects.get()
        ctxt.tuned_temperature = ctxt.confort_temperature
        ctxt.save()
        logger.debug("Tunned to comfort")
    except Exception as et:
        logger.error(et)


def tune_to_economic(mo=None):
    try:

        ctxt = Context.objects.get()

        ctxt.tuned_temperature = ctxt.economic_temperature
        ctxt.save()
        logger.debug("Tunned to economic")
    except Exception as et:
        logger.error(et)
    


def heater_manual(mo=None):
    try:
        r = int(Context.objects.get().manual)
        logger.debug("Heater is set to manual: %s" % r)
        return r
    except Exception as et:
        logger.error(et)


def heater_on(mo=None):
    try:
        r = int(Context.objects.get().heat_on)
        logger.debug("Heater is on: %s" % r)
        return r
    except Exception as et:
        logger.error(et)


def set_heater_on(no=None):
    try:
        ctxt = Context.objects.get()
        ctxt.heat_on = True
        ctxt.save()
        logger.debug("Heater tuner ON")
    except Exception as et:
        logger.error(et)


def set_heater_off(no=None):
    try:
        ctxt = Context.objects.get()
        ctxt.heat_on = False
        ctxt.save()
        logger.debug("Heater turned OFF")
    except Exception as et:
        logger.error(et)

def flame_on():
    try:
        r = 1 if Context.objects.get().flame else 0
        logger.debug("Flamie is on: %s" % r)
        return r
    except Exception as et:
        logger.error(et)

def log_flame_stats(new_state):
    try:
        if settings.FLAME_STATS:
            with open(settings.FLAME_STATS_PATH, "a") as stats:
                t = localtime()
                st = strftime(settings.FLAME_STATS_DATE_FORMAT, t)
                stats.write("%s - %s - %f\n" % (
                    "ON" if new_state else "OFF",
                    st,
                    mktime(strptime(st, settings.FLAME_STATS_DATE_FORMAT))))
    except Exception as et:
        logger.error(et)


def start_flame():

    try:
        ctxt = Context.objects.get()
        if ctxt.flame:
            logger.debug("Not starting flame, because its already started")
            return

        pl_switch(
            settings.HEATER_PROTOCOL,
            settings.HEATER_DID,
            "on",
            settings.HEATER_API,
            settings.HEATER_USERNAME,
            settings.HEATER_PASSWORD)

        ctxt.flame = True
        ctxt.save()

        logger.debug("Flame started")
        log_flame_stats(True)
    except Exception as et:
        logger.error(et)

def stop_flame():
    try:
        ctxt = Context.objects.get()
        if not ctxt.flame:
            logger.debug("Not stopping flame, because its already stopped")
            return

        pl_switch(
            settings.HEATER_PROTOCOL,
            settings.HEATER_DID,
            "off",
            settings.HEATER_API,
            settings.HEATER_USERNAME,
            settings.HEATER_PASSWORD)

        ctxt.flame = False
        ctxt.save()

        logger.debug("Flame stopped")
        log_flame_stats(False)
        #print "%s flame stopped" % strftime("%d.%m.%Y %H:%M:%S", localtime())
    except Exception as et:
        logger.error(et)

mappings = [
    current_internal_temperature,
    current_external_temperature,
    confort_temperature,
    economic_temperature,
    start_flame,
    stop_flame,
    heater_manual,
    heater_on,
    flame_on,
    tune_to_confort,
    tune_to_economic,
    tuned_temperature,
    set_heater_off,
    set_heater_on,
    ]



import logging

from hautomation_restclient.cmds import (
    pl_switch,
    pl_all_lights_off,
    pl_all_lights_on)


from django.conf import settings

logger = logging.getLogger("thermostat")
logger.setLevel(settings.LOG_LEVEL)


def salon_on(mo=None):
    try:
        pl_switch(
            settings.HEATER_PROTOCOL,
            settings.SALON_DID,
            "on",
            settings.HEATER_API,
            settings.HEATER_USERNAME,
            settings.HEATER_PASSWORD)
        logger.debug("Salon subido")
    except Exception as ex:
        logger.error(ex)


def salon_off(mo=None):
    try:
        pl_switch(
            settings.HEATER_PROTOCOL,
            settings.SALON_DID,
            "off",
            settings.HEATER_API,
            settings.HEATER_USERNAME,
            settings.HEATER_PASSWORD)
        logger.debug("Salon bajado")
    except Exception as ex:
        logger.error(ex)


def pasillo_off(mo=None):
    try:
        pl_switch(
            settings.HEATER_PROTOCOL,
            settings.PASILLO_DID,
            "off",
            settings.HEATER_API,
            settings.HEATER_USERNAME,
            settings.HEATER_PASSWORD)
        logger.debug("Pasillo bajado")
    except Exception as ex:
        logger.error(ex)


def pasillo_on(mo=None):
    try:
        pl_switch(
            settings.HEATER_PROTOCOL,
            settings.PASILLO_DID,
            "on",
            settings.HEATER_API,
            settings.HEATER_USERNAME,
            settings.HEATER_PASSWORD)
        logger.debug("Pasillo subido")
    except Exception as ex:
        logger.error(ex)


def cuarto_oeste_off(mo=None):

    try:
        pl_switch(
            settings.HEATER_PROTOCOL,
            settings.CUARTO_OESTE_DID,
            "off",
            settings.HEATER_API,
            settings.HEATER_USERNAME,
            settings.HEATER_PASSWORD)
        logger.debug("Cuarto oeste bajado")
    except Exception as ex:
        logger.error(ex)


def cuarto_oeste_on(mo=None):
    try:
        pl_switch(
            settings.HEATER_PROTOCOL,
            settings.CUARTO_OESTE_DID,
            "on",
            settings.HEATER_API,
            settings.HEATER_USERNAME,
            settings.HEATER_PASSWORD)
        logger.debug("Cuarto oeste subido")
    except Exception as ex:
        logger.error(ex)


def cuarto_este_off(mo=None):
    try:
        pl_switch(
            settings.HEATER_PROTOCOL,
            settings.CUARTO_ESTE_DID,
            "off",
            settings.HEATER_API,
            settings.HEATER_USERNAME,
            settings.HEATER_PASSWORD)
        logger.debug("Cuarto este bajado")
    except Exception as ex:
        logger.error(ex)

def cuarto_este_on(mo=None):
    try:
        pl_switch(
            settings.HEATER_PROTOCOL,
            settings.CUARTO_ESTE_DID,
            "on",
            settings.HEATER_API,
            settings.HEATER_USERNAME,
            settings.HEATER_PASSWORD)
        logger.debug("Cuarto este subido")
    except Exception as ex:
        logger.error(ex)


def a_lights_off(mo=None):
    try:
        pl_all_lights_off(
            settings.HEATER_PROTOCOL,
            "A",
            settings.HEATER_API,
            settings.HEATER_USERNAME,
            settings.HEATER_PASSWORD)
        logger.debug("Grupo A bajado")
    except Exception as ex:
        logger.error(ex)


def a_lights_on(mo=None):
    try:
        pl_all_lights_on(
            settings.HEATER_PROTOCOL,
            "A",
            settings.HEATER_API,
            settings.HEATER_USERNAME,
            settings.HEATER_PASSWORD)
        logger.debug("Grupo A subido")
    except Exception as ex:
        logger.error(ex)


mappings = [
    salon_off,
    salon_on,
    cuarto_este_on,
    cuarto_este_off,
    cuarto_oeste_off,
    cuarto_oeste_on,
    pasillo_on,
    pasillo_off,
    a_lights_off,
    a_lights_on,
    ]



from time import strftime, localtime, mktime, strptime
from astral import Location#, AstralGeocoder
import datetime, logging
from django.conf import settings as settings
import pytz


logger = logging.getLogger("thermostat")
logger.setLevel(settings.LOG_LEVEL)


def current_day_of_week(mo=None):
    try:
        d = strftime("%a", localtime())
        logger.debug("Current day of week: %s" % d)
        return d
    except Exception as ex:
        logger.error(ex)


def current_month(mo=None):
    try:
        m = strftime("%m", localtime())
        logger.debug("Current month: %s" % m)
        return m
    except Exception as ex:
        logger.error(ex)


def current_year(mo=None):
    try:
        y = strftime("%Y", localtime())
        logger.debug("Current year: %s" % y)
        return y
    except Exception as ex:
        logger.error(ex)


def current_day_of_month(mo=None):
    try:
        d = strftime("%d", localtime())
        logger.debug("Current day of month: %s" % d)
        return d
    except Exception as ex:
        logger.error(ex)


def current_time(mo=None):
    try:
        lt = localtime()
        st = "%s %s %s %s:%s:%s" %(
            strftime("%d", lt),
            strftime("%m", lt),
            strftime("%Y", lt),
            strftime("%H", lt),
            strftime("%M", lt),
            strftime("%S", lt))
        t = strptime(st, "%d %m %Y %H:%M:%S")
        t = mktime(t)
        logger.debug("Current time: %s" % t)
        return t
    except Exception as ex:
        logger.error(ex)



def is_weekend(mo=None):
    try:
        today = current_day_of_week()
        if today == "Sat" or today == "Sun":
            logger.debug("Is not weekend")
            return 1
        logger.debug("Is weekend")
        return 0
    except Exception as ex:
        logger.error(ex)


def is_at_night(mo=None):
    try:
        a = Location()
        a.timezone = dsettings.TIME_ZONE
        tz = pytz.timezone(a.timezone)
        #Tue, 22 Jul 2008 08:17:41 +0200
        #Sun, 26 Jan 2014 17:39:49 +01:00
        a_sunset = a.sunset()

        a_sunrise = a.sunrise()

        n = datetime.datetime.now()
        n = tz.localize(n)
        logger.debug("NOW: %s; sunrise: %s; dif: %s"  % (n, a_sunrise, n - a_sunrise))
        logger.debug("NOW: %s; sunset: %s; dif: %s" % (n, a_sunset, n - a_sunset))
        passed_sunrise = (n - a_sunrise) > datetime.timedelta(minutes=settings.MINUTES_AFTER_SUNRISE_FOR_DAY)
        logger.debug("Passed %s sunrise more than %s minutes" % (passed_sunrise, settings.MINUTES_AFTER_SUNRISE_FOR_DAY))
        passed_sunset = (n - a_sunset) > datetime.timedelta(minutes=settings.MINUTES_AFTER_SUNSET_FOR_DAY)
        logger.debug("Passed %s sunset more than %s minutes" % (passed_sunset, settings.MINUTES_AFTER_SUNSET_FOR_DAY))

        if not passed_sunrise or passed_sunset:
            logger.debug("Is not at night")
            return 1
        if passed_sunrise and not passed_sunset:
            logger.debug("Is at night")
            return 0
    except Exception as ex:
        logger.error(ex)

mappings = [
    current_day_of_week,
    current_time,
    is_weekend,
    current_month,
    current_day_of_month,
    current_year,
    is_at_night,
    ]
